package mackthehobbit.ld26;

import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;

import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JPanel;

import mackthehobbit.ld26.entity.PlayerEntity;
import mackthehobbit.ld26.gfx.Screen;
import mackthehobbit.ld26.item.Inventory;
import mackthehobbit.ld26.map.Map;

public class Game extends Canvas {
	private static final long serialVersionUID = 1L;

	public static final int WIDTH = 160;
	public static final int HEIGHT = 112;
	public static final int SCALE = 4;
	public static final String TITLE = "Ludum Dare 26 : Hatchet";
	
	public static Game g;
	public static Screen screen;
	public static Map map;
	public static Inventory inventory = new Inventory();
	public static KeyInputer input = new KeyInputer();
	
	public static int tickCount = 0;
	
	public static final int GOALFPS = 32;
	private static final int skipTicks = 1000/GOALFPS;
	public static boolean makingFps = true;
	private long nextTick = System.currentTimeMillis();
	
	public boolean running = true;
	public boolean inMenu = false;

	private String notifyMsg = "Hello, minimal world.";
	private int timeSinceNotify = 64;
	
	public void gameLoop() {
		Intro intro = new Intro();
		while(intro.running()) {
			System.out.println("Ticking intro");
			intro.tick();
			screen.tick();
			doTickFps();
		}
		
		while(running) {
			doLoop();
		}
	}
	
	public void doLoop() {
		Game.tickCount++;
		if(!inMenu) {
			timeSinceNotify--;
			if(timeSinceNotify<=0) {
				notifyMsg="";
				timeSinceNotify=0;
			}
			if(input.s.isPressed()) {
				inMenu = true;
				CraftMenu.prepare();
			}
			map.tick();
		}
		else {
			map.renderAllTiles(screen);
			CraftMenu.tick();
			CraftMenu.render();
			if(!CraftMenu.isOpen) {
				inMenu = false;
			}
		}
		renderGui();
		screen.tick();
		doTickFps();
	}
	
	public void doTickFps() {
		nextTick += skipTicks;
		long sleepTime = nextTick - System.currentTimeMillis();
		if( sleepTime >= 0 ) {
			makingFps = true;
			try {
				Thread.sleep(sleepTime);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		else {
			//Oh poop. We're not getting our target FPS.
			makingFps = false;
		}
	}
	
	private void renderGui() {
		screen.renderRectangle(0, 0, WIDTH, 16);
		screen.renderRectangle(0, HEIGHT-8, WIDTH, HEIGHT);
		screen.renderText(0, 0, "HATCHET: LD26", 2);
		screen.renderText(0, 32, notifyMsg, 2);
		String fpsString = "AIM FPS:"+GOALFPS;
		screen.renderText(WIDTH*2 - fpsString.length()*8, 0, fpsString, 2);
		if(!makingFps) screen.renderText(WIDTH*2-"BEHIND!".length()*8, 8, "BEHIND!", 2);
		screen.renderText(0, 8, "Z", 1);
		screen.renderText(32+2, 8, "X", 1);
		screen.renderText(64+4, 8, "C", 1);
		screen.renderText(96+6, 8, "V", 1);
		screen.renderText(128+8, 8, "B", 1);
		
		for(int i=0; i < 5; i++) {
			inventory.getItem(i).getItem().render(screen, i*34+8, 8);
			if(inventory.getItem(i).size > 1) {
				screen.renderText(i*34+16, 8, ""+inventory.getItem(i).size, 1);
			}
		}
		screen.renderText(WIDTH/2-8, 0, "S", 1);
		screen.render(WIDTH/2, 0, 1 * 8, 4 * 8, 8, 8);		
		
		screen.render(0, HEIGHT-8, 0 * 8, 2 * 8, 8, 8);
		screen.renderText(8, HEIGHT-8, ""+Game.inventory.woodCount, 1);
		screen.render(40, HEIGHT-8, 1 * 8, 2 * 8, 8, 8);
		screen.renderText(48, HEIGHT-8, ""+Game.inventory.stoneCount, 1);
		screen.render(80, HEIGHT-8, 3 * 8, 2 * 8, 8, 8);
		screen.renderText(88, HEIGHT-8, ""+Game.inventory.goldCount, 1);
		screen.render(120, HEIGHT-8, 0 * 8, 5 * 8, 8, 8);
		screen.renderText(128, HEIGHT-8, ""+map.player.health, 1);
	}

	
	public void notify(String msg) {
		this.notifyMsg  = msg;
		this.timeSinceNotify = 128;
	}
	
	
	public static void main(String[] args) {
		g = new Game();
		
		JFrame frame = new JFrame(TITLE);
		JPanel panel = (JPanel) frame.getContentPane();
		
		panel.setPreferredSize(new Dimension(WIDTH * SCALE - 10, HEIGHT * SCALE - 10));
		frame.add(g);
		g.setIgnoreRepaint(true);
		frame.pack();
		frame.setLocationRelativeTo(null);
		frame.setResizable(false);
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		g.addKeyListener(input);
		g.requestFocus();
		g.createBufferStrategy(2);
		Game.screen = new Screen(g.getBufferStrategy());
		Game.map = new Map();
		Game.map.addEntity(new PlayerEntity(Game.map.spawnX, Game.map.spawnY));
		
		try {
			URL url = g.getClass().getClassLoader().getResource("res/icon.png");
		    BufferedImage icon = ImageIO.read(url);
		    frame.setIconImage(icon);
		} catch (IOException e) {
		    e.printStackTrace();
		}
		
		g.gameLoop();
	}

	public static void playerDie() {
		map.renderAllTiles(screen);
		g.renderGui();
		screen.renderText(WIDTH/2-"You have lost!".length()/2*8, HEIGHT/2-8, "You have lost!", 1);
		screen.renderText(WIDTH-"Press esc to exit.".length()/2*8, HEIGHT+4, "Press esc to exit.", 2);
		screen.tick();
		while(true) {}
	}
	
	public static void playerWin() {
		for(;;) {
			g.doTickFps();
			map.tick();
			g.renderGui();
			screen.renderText(WIDTH/2-"You win C:".length()/2*8, HEIGHT/2-8, "You win C:", 1);
			screen.renderText(WIDTH-"Press esc to exit, or z to keep playing.".length()/2*8, HEIGHT+4, "Press esc to exit, or z to keep playing.", 2);
			screen.tick();
			if(input.z.isPressed()) break;
		}
	}
	
}
