package mackthehobbit.ld26.entity;

import mackthehobbit.ld26.Game;
import mackthehobbit.ld26.Sound;
import mackthehobbit.ld26.gfx.Screen;
import mackthehobbit.ld26.map.Tile;

public class PlayerEntity extends Entity {
	private int walkDist;
	public int dir;
	public int interactTimer = 0;
	public int hurtTimer;
	
	public int health = 80;
	
	public PlayerEntity(int x, int y) {
		super(x, y);
	}
	
	public void move(int xm, int ym) {
		if(!Game.map.getTileAt((x+xm)/16, (y+ym)/16).isSolid() && !((x+xm)/16 <= 0 || (y+ym)/16 <= 0 || (x+xm)/16 >= Game.map.width || (y+ym)/16 >= Game.map.height))
			super.move(xm, ym);
		else {
			Sound.bump.play();
		}
	}
	
	public void tick() {
		super.tick();
		interactTimer--;
		hurtTimer--;
		if(tickCount % (Game.map.getTileAt(x/16, y/16)==Tile.water?2:1)==0) doInput();
		if(walkDist>=10) walkDist = 0;
		 
		if(Game.input.z.isDown()) {
			Game.inventory.use(0);
		}
		if(Game.input.x.isDown()) {
			Game.inventory.use(1);
		}
		if(Game.input.c.isDown()) {
			Game.inventory.use(2);
		}
		if(Game.input.v.isDown()) {
			Game.inventory.use(3);
		}
		if(Game.input.b.isDown()) {
			Game.inventory.use(4);
		}
	}
	
	private void doInput() {
		int prevX = x;
		int prevY = y;
		
		if(Game.input.down.isDown()) {
			move(0, 1);
			dir = 0;
		}
		if(Game.input.up.isDown()) {
			move(0, -1);
			dir = 1;
		}
		if(Game.input.right.isDown()) {
			move(1, 0);
			dir = 2;
		}
		if(Game.input.left.isDown()) {
			move(-1, 0);
			dir = 3;
		}
		
		walkDist += Math.sqrt((x-prevX)*(x-prevX)) + Math.sqrt((y-prevY)*(y-prevY)) >= 1? 1:0;
	}
	
	public void render(Screen screen, int scrollX, int scrollY) {
		boolean inWater = Game.map.getTileAt(x/16, y/16)==Tile.water;
		screen.render(x - 4 - scrollX, y - 8 - scrollY + (inWater?4:0), 0, walkDist/5 * 8, 8, (inWater?4:8));
	}
	
	public void heal(int amount) {
		health+=amount;
		health=health>=100? 100:health;
		Game.g.notify("Restored "+amount+" health.");
	}
	
	public void hurt(int amount) {
		if(hurtTimer>0) return;
		Sound.hurt.play();
		health-=amount;
		hurtTimer = 32;
		if(health<=0) {
			die();
		}
	}
	
	public void die() {
		Game.map.removeEntity(this);
		Game.playerDie();
	}

}
