package mackthehobbit.ld26.gfx;

import java.applet.Applet;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferStrategy;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;

import javax.imageio.ImageIO;

import mackthehobbit.ld26.Game;

public class Screen {
	private BufferedImage sheet;
	private BufferedImage fontsheet;
	
	private BufferStrategy bstrategy;
	private Graphics2D g;
	
	public boolean appletMode = false;
	private Applet applet;
	
	public boolean isMax = false;
	
	public Screen(Applet applet) {
		this.applet = applet;
		appletMode = true;
		
		this.g = (Graphics2D) applet.getGraphics();
		
		sheet = loadSheet("res/sheet.png");
		fontsheet = loadSheet("res/fontsheet.png");
	}
	
	public Screen(BufferStrategy bstrategy) {
		this.bstrategy = bstrategy;
		
		g = (Graphics2D) bstrategy.getDrawGraphics();
		
		sheet = loadSheet("res/sheet.png");
		fontsheet = loadSheet("res/fontsheet.png");
	}
	
	public BufferedImage loadSheet(String path) {
		try {
			URL url = this.getClass().getClassLoader().getResource(path);
			if(url==null) throw new IOException();
			return ImageIO.read(url);
		} catch(IOException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public void maximalise() {
		this.sheet = loadSheet("res/sheet_maximalised.png");
		isMax = true;
	}
	public void minimalise() {
		this.sheet = loadSheet("res/sheet.png");
		isMax = false;
	}
	
	public void tick() {
		g.dispose();
		
		
		if(appletMode) {
			applet.setVisible(true);
			g = (Graphics2D) applet.getGraphics();
			g.setColor(Color.black);
			g.fillRect(0, 0, Game.WIDTH * Game.SCALE, Game.HEIGHT * Game.SCALE);
		}
		else {
			bstrategy.show();
			g = (Graphics2D) bstrategy.getDrawGraphics();
			g.setColor(Color.black);
			g.fillRect(0, 0, Game.WIDTH * Game.SCALE, Game.HEIGHT * Game.SCALE);
		}
	}
	
	public void render(int xp, int yp, int tileX, int tileY, int tileW, int tileH) {
		BufferedImage tile = sheet.getSubimage(tileX, tileY, tileW, tileH);
		g.drawImage(tile, xp * Game.SCALE, yp * Game.SCALE, tileW * Game.SCALE, tileH * Game.SCALE, null);
	}
	
	public void renderText(int x, int y, String text, int scaleDown) {
		text = text.toUpperCase();
		String ln1="ABCDEFGHIJKLMNOP";
		String ln2="QRSTUVWXYZ012345";
		String ln3="6789.,$#>!?'" + '"' + "_: ";
		int xd = -1, yd = -1;
		for(int c=0; c < text.length(); c++) {
			int i = ln1.indexOf(text.charAt(c));
			if (i != -1) {
				yd = 0;
				xd = ln1.indexOf(text.charAt(c));
			}
			i = ln2.indexOf(text.charAt(c));
			if (i != -1) {
				yd = 1;
				xd = ln2.indexOf(text.charAt(c));
			}
			i = ln3.indexOf(text.charAt(c));
			if (i != -1) {
				yd = 2;
				xd = ln3.indexOf(text.charAt(c));
			}
			if(xd > -1 && yd > -1) {
				BufferedImage character = fontsheet.getSubimage(xd * 8, yd * 8, 8, 8);
				g.drawImage(character, (x + c*8) * Game.SCALE/scaleDown, y * Game.SCALE/scaleDown, character.getWidth(null)*Game.SCALE/scaleDown, character.getHeight(null)*Game.SCALE/scaleDown, null);
			}
		}
	}
	
	public void renderRectangle(int x, int y, int width, int height) {
		g.setColor(Color.black);
		g.fillRect(x*Game.SCALE, y*Game.SCALE, width*Game.SCALE, height*Game.SCALE);
	}
	
}
