package mackthehobbit.ld26.item;

import java.util.Random;

import mackthehobbit.ld26.Game;
import mackthehobbit.ld26.Sound;
import mackthehobbit.ld26.entity.PlayerAttack;
import mackthehobbit.ld26.entity.SparklySparklyParticle;
import mackthehobbit.ld26.gfx.Screen;
import mackthehobbit.ld26.map.PotatoTile;
import mackthehobbit.ld26.map.Tile;
import mackthehobbit.ld26.map.TraderNPCTile;

public class Item {
	public static Item hatchet = new Item("Hatchet", 99, 0 * 8, 4 * 8);
	public static Item wall = new Item("Wall", 99, 0 * 8, 3 * 8);
	public static Item shovel = new Item("Shovel", 99, 1 * 8, 3 * 8);
	public static Item potato = new Item("Potato", 99, 2 * 8, 3 * 8);
	public static Item meat = new Item("Meat", 99, 3 * 8, 3 * 8);
	public static Item max = new Item("Maximaliser", 10, 2 * 8, 4 * 8);
	public static Item none = new Item("EMPTY", 1, 3 * 8, 5 * 8);
	
	public String name;
	public int maxSize;
	public int tileX, tileY;
	
	private Random random = new Random();
	
	public Item(String name, int maxSize, int tileX, int tileY) {
		this.name = name;
		this.maxSize = maxSize;
		this.tileX = tileX;
		this.tileY = tileY;
	}
	
	public void render(Screen screen, int x, int y) {
		screen.render(x, y, tileX, tileY, 8, 8);
	}
	
	public boolean use() {
		Game.map.player.interactTimer = 5;
		int tileX = Game.map.player.getTileX();
		int tileY = Game.map.player.getTileY();
		if(Game.map.player.dir==0 && Game.map.player.y % 16 >= 8) tileY++;
		if(Game.map.player.dir==1 && Game.map.player.y % 16 <= 8) tileY--;
		if(Game.map.player.dir==2 && Game.map.player.x % 16 >= 8) tileX++;
		if(Game.map.player.dir==3 && Game.map.player.x % 16 <= 8) tileX--;
		Tile tile = Game.map.getTileAt(tileX, tileY);
		
		if(tile instanceof TraderNPCTile) {
			((TraderNPCTile) tile).tryTrade(this);
			return false;
		}
		int ax = Game.map.player.x;
		int ay = Game.map.player.y;
		if(Game.map.player.dir==0) ay+=6;
		if(Game.map.player.dir==1) ay-=6;
		if(Game.map.player.dir==2) ax+=6;
		if(Game.map.player.dir==3) ax-=6;
		
		if(this==hatchet) {
			Game.map.addEntity(new PlayerAttack(ax, ay, 4));
			if(tile==Tile.stump || tile==Tile.wall) {
				Sound.rumble.play();
				Game.map.setTileAt(tileX, tileY, Tile.ground);
				Game.inventory.addWood(1);
			}
			if(tile==Tile.tree) {
				Sound.hit.play();
				Game.map.setTileAt(tileX, tileY, Tile.stump);
				Game.inventory.addWood(random.nextInt(2));
			}
		}
		
		if(this==wall) {
			if(tile==Tile.ground && (tileX!=Game.map.player.getTileX() || tileY!=Game.map.player.getTileY())) {
				Sound.bump.play();
				Game.map.setTileAt(tileX, tileY, Tile.wall);
				return true;
			}
		}
		
		if(this==shovel) {
			Game.map.addEntity(new PlayerAttack(ax, ay, 3));
			if(tile==Tile.ground) {
				Sound.dig.play();
				Game.map.setTileAt(tileX, tileY, Tile.pit);
				if(random.nextInt(8)==0) {
					Game.inventory.addStone(1);
				}
				else if(random.nextInt(40)==0) {
					Game.inventory.addItem(new ItemStore(Item.potato, 1));
				}
				else if(random.nextInt(100)==0) {
					Game.inventory.addGold(1);
				}
			}
			if(tile instanceof PotatoTile) {
				Game.inventory.addItem(new ItemStore(Item.potato, ((PotatoTile)tile).stage+1));
				Game.map.setTileAt(tileX, tileY, Tile.ground);
			}
		}
		
		if(this==potato) {
			if(Game.map.player.health<100) {
				Game.map.player.heal(15);
				return true;	
			}
			else if(tile==Tile.ground) {
				Game.map.setTileAt(tileX, tileY, Tile.potato1);
				return true;
			}
		}
		if(this==meat) {
			if(Game.map.player.health<100) {
				Game.map.player.heal(25);
				return true;	
			}
		}
		if(this==max) {
			Game.screen.maximalise();
			for(int i=0;i<random.nextInt(1024)+512; i++) {
				Game.map.addEntity(new SparklySparklyParticle(Game.map.scrollX+random.nextInt(Game.WIDTH), Game.map.scrollY+random.nextInt(Game.HEIGHT)));
			}
			Sound.max.play();
			Game.inventory.removeItem(Item.max, 1);
			Game.playerWin();
		}
		return false;
	}

}
