package mackthehobbit.ld26.map;

import java.util.ArrayList;
import java.util.Random;

import mackthehobbit.ld26.Game;
import mackthehobbit.ld26.entity.Entity;
import mackthehobbit.ld26.entity.PlayerEntity;
import mackthehobbit.ld26.entity.ZombieEntity;
import mackthehobbit.ld26.gfx.Screen;

public class Map {
	private int screenWidth = Game.WIDTH/16;
	private int screenHeight = Game.HEIGHT/16;
	
	public int width = 128;
	public int height = 96;
	
	public int scrollX = 0;
	public int scrollY = 0;
	public PlayerEntity player;
	public int zombieCount;
	
	public int spawnX, spawnY;
	
	private Random random = new Random();
	
	private Tile[][] tileData = new Tile[width][height];
	private ArrayList<Entity> entities = new ArrayList<Entity>();
	private ArrayList<Entity>[][] entityTiles = new ArrayList[width][height];
	
	public Map() {
		new MapGen(this);
		this.scrollX = spawnX-Game.WIDTH/2;
		this.scrollY = spawnY-Game.HEIGHT/2;
		for(int xt=0; xt < width; xt++) {
			for(int yt=0; yt < height; yt++) {
				entityTiles[xt][yt] = new ArrayList<Entity>();
			}
		}
	}
	
	public void tick() {
		for (int i=0;i<entities.size();i++) {
			Entity entity = entities.get(i);	
			entity.tick();
		}
		for (int i=0;i<entities.size();i++) {
			for(int j=0; j<entities.size(); j++) {
				if(entities.get(j).collidesWith(entities.get(i))) {
					entities.get(j).collideWith(entities.get(i));
				}
			}
		}
		
		if(zombieCount < 500 && random.nextInt(32)==0) {
			int xg = random.nextInt(width);
			int yg = random.nextInt(height);
			if(getTileAt(xg, yg)!=Tile.water&&!getTileAt(xg, yg).isSolid()) {
				if(xg < player.x-Game.WIDTH/2||xg > player.x+Game.WIDTH||yg < player.y-Game.HEIGHT/2||yg > player.y+Game.HEIGHT/2) {
					addEntity(new ZombieEntity(xg*16 + 8, yg*16 + 8));
				}
			}
		}
		
		if(player!=null) {
			if(player.x > scrollX + Game.WIDTH/2) {
				scrollX++;
			}
			if(player.x < scrollX + Game.WIDTH/2) {
				scrollX--;
			}
			if(player.y > scrollY + Game.HEIGHT/2) {
				scrollY++;
			}
			if(player.y < scrollY + Game.HEIGHT/2) {
				scrollY--;
			}
		}
		
		/*
		if(Game.input.down.isDown()) scrollY++;
		if(Game.input.up.isDown()) scrollY--;
		if(Game.input.right.isDown()) scrollX++;
		if(Game.input.left.isDown()) scrollX--;
		*/
		
		renderAllTiles(Game.screen);
	}
	
	public void renderAllTiles(Screen screen) {
		for(int yt=0; yt < height; yt++) {
			for(int xt=0; xt < width; xt++) {
				if(xt*16 > scrollX-16 && yt*16 > scrollY-16 && xt*16 <= scrollX+Game.WIDTH && yt*16 <= scrollY+Game.HEIGHT) {
					getTileAt(xt, yt).render(screen, xt * 16 - scrollX, yt * 16 - scrollY);
				}
				if(random.nextInt(1600)==0) {
					getTileAt(xt, yt).tick(xt, yt);
				}
			}
			for(int xt=0; xt < width; xt++) {
				ArrayList<Entity> es = entityTiles[xt][yt];
				for(int i=0; i < es.size(); i++) {
					es.get(i).render(screen, scrollX, scrollY);
				}
			}
		}
		
	}
	
	public Tile getTileAt(int xt, int yt) {
		if (xt >= 0 && yt >= 0 && xt < width && yt < height)
			return tileData[xt][yt];
		else return Tile.water;
	}
	
	public void setTileAt(int xt, int yt, Tile tile) {
		if (xt >= 0 && yt >= 0 && xt < width && yt < height) {
			tileData[xt][yt] = tile;
		}
	}
	
	public int getEntityTileX(Entity e) {
		for(int x=0; x<width; x++) {
			for(int y=0; y<height; y++) {
				if(entityTiles[x][y].contains(e)) {
					return x;
				}
			}
		}
		return -1;
	}
	
	public int getEntityTileY(Entity e) {
		for(int x=0; x<width; x++) {
			for(int y=0; y<height; y++) {
				if(entityTiles[x][y].contains(e)) {
					return y;
				}
			}
		}
		return -1;
	}
	
	public void setEntityTile(int xt, int yt, Entity e) {
		for(int x=0; x<width; x++) {
			for(int y=0; y<height; y++) {
				if(entityTiles[x][y].contains(e)) {
					entityTiles[x][y].remove(e);
				}
			}
		}
		entityTiles[xt][yt].add(e);
	}

	
	public void addEntity(Entity entity) {
		entities.add(entity);
		if(entity instanceof PlayerEntity) {
			this.player = (PlayerEntity) entity;
		}
		if(entity instanceof ZombieEntity) {
			zombieCount++;
		}
	}
	
	public void removeEntity(Entity entity) {
		entities.remove(entity);
		for(int x=0; x<width; x++) {
			for(int y=0; y<height; y++) {
				if(entityTiles[x][y].contains(entity)) {
					entityTiles[x][y].remove(entity);
				}
			}
		}
		if(entity instanceof ZombieEntity) {
			zombieCount--;
		}
	}
	
}
